var _createAssigner = require('./_createAssigner.js');
var allKeys = require('./allKeys.js');

// Extend a given object with all the properties in passed-in object(s).
var extend = _createAssigner(allKeys);

module.exports = extend;
