import parseLinkLabel = require('./parse_link_label');
import parseLinkDestination = require('./parse_link_destination');
import parseLinkTitle = require('./parse_link_title');

interface Helpers {
    parseLinkLabel: typeof parseLinkLabel;
    parseLinkDestination: typeof parseLinkDestination;
    parseLinkTitle: typeof parseLinkTitle;
}

declare const helpers: Helpers;

export = helpers;
