define(['./_createEscaper', './_unescapeMap'], function (_createEscaper, _unescapeMap) {

	// Function for unescaping strings from HTML interpolation.
	var _unescape = _createEscaper(_unescapeMap);

	return _unescape;

});
