var _createEscaper = require('./_createEscaper.js');
var _unescapeMap = require('./_unescapeMap.js');

// Function for unescaping strings from HTML interpolation.
var _unescape = _createEscaper(_unescapeMap);

module.exports = _unescape;
