declare namespace parseLinkDestination {
    interface ParseResult {
        ok: boolean;
        pos: number;
        lines: number;
        str: string;
    }
}

declare function parseLinkDestination(str: string, pos: number, max: number): parseLinkDestination.ParseResult;

export = parseLinkDestination;
