// Type definitions for markdown-it 12.2
// Project: https://github.com/markdown-it/markdown-it
// Definitions by: York Yao <https://github.com/plantain-00>
//                 Robert Coie <https://github.com/rapropos>
//                 duduluu <https://github.com/duduluu>
//                 Piotr Błażejewicz <https://github.com/peterblazejewicz>
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped

import MarkdownIt = require('./lib');

export as namespace markdownit;

export = MarkdownIt;
